@@ -85,6 +85,19 @@ public:
 		scDeactivateServer,
 		scFetchFeed
 	};
+	enum EPostStrategy
+	{
+		ppSequential,
+		ppBalanced,
+		ppAggressive,
+		ppRocket
+	};
+	enum EFileNaming
+	{
+		nfAuto,
+		nfArticle,
+		nfNzb
+	};
 
 	class OptEntry
 	{
@@ -126,19 +139,19 @@ public:
 	class Category
 	{
 	public:
-		Category(const char* name, const char* destDir, bool unpack, const char* postScript) :
-			m_name(name), m_destDir(destDir), m_unpack(unpack), m_postScript(postScript) {}
+		Category(const char* name, const char* destDir, bool unpack, const char* extensions) :
+			m_name(name), m_destDir(destDir), m_unpack(unpack), m_extensions(extensions) {}
 		const char* GetName() { return m_name; }
 		const char* GetDestDir() { return m_destDir; }
 		bool GetUnpack() { return m_unpack; }
-		const char* GetPostScript() { return m_postScript; }
+		const char* GetExtensions() { return m_extensions; }
 		NameList* GetAliases() { return &m_aliases; }
 
 	private:
 		CString m_name;
 		CString m_destDir;
 		bool m_unpack;
-		CString m_postScript;
+		CString m_extensions;
 		NameList m_aliases;
 	};
 
@@ -154,12 +167,12 @@ public:
 	{
 	public:
 		virtual void AddNewsServer(int id, bool active, const char* name, const char* host,
-			int port, const char* user, const char* pass, bool joinGroup,
+			int port, int ipVersion, const char* user, const char* pass, bool joinGroup,
 			bool tls, const char* cipher, int maxConnections, int retention,
 			int level, int group, bool optional) = 0;
 		virtual void AddFeed(int id, const char* name, const char* url, int interval,
 			const char* filter, bool backlog, bool pauseNzb, const char* category,
-			int priority, const char* feedScript) {}
+			int priority, const char* extensions) {}
 		virtual void AddTask(int id, int hours, int minutes, int weekDaysBits, ESchedulerCommand command,
 			const char* param) {}
 		virtual void SetupFirstStart() {}
@@ -170,9 +183,12 @@ public:
 	Options(CmdOptList* commandLineOptions, Extender* extender);
 	~Options();
 
-	bool SplitOptionString(const char* option, CString& optName, CString& optValue);
+	static bool SplitOptionString(const char* option, CString& optName, CString& optValue);
+	static void ConvertOldOptions(OptEntries* optEntries);
 	bool GetFatalError() { return m_fatalError; }
 	GuardedOptEntries GuardOptEntries() { return GuardedOptEntries(&m_optEntries, &m_optEntriesMutex); }
+	void CreateSchedulerTask(int id, const char* time, const char* weekDays,
+		ESchedulerCommand command, const char* param);
 
 	// Options
 	const char* GetConfigFilename() { return m_configFilename; }
@@ -197,11 +213,14 @@ public:
 	int GetArticleTimeout() { return m_articleTimeout; }
 	int GetUrlTimeout() { return m_urlTimeout; }
 	int GetTerminateTimeout() { return m_terminateTimeout; }
+	int GetRemoteTimeout() { return m_remoteTimeout; }
 	bool GetDecode() { return m_decode; };
 	bool GetAppendCategoryDir() { return m_appendCategoryDir; }
 	bool GetContinuePartial() { return m_continuePartial; }
-	int GetRetries() { return m_retries; }
-	int GetRetryInterval() { return m_retryInterval; }
+	int GetArticleRetries() { return m_articleRetries; }
+	int GetArticleInterval() { return m_articleInterval; }
+	int GetUrlRetries() { return m_urlRetries; }
+	int GetUrlInterval() { return m_urlInterval; }
 	bool GetSaveQueue() { return m_saveQueue; }
 	bool GetFlushQueue() { return m_flushQueue; }
 	bool GetDupeCheck() { return m_dupeCheck; }
@@ -213,10 +232,13 @@ public:
 	const char* GetAddUsername() { return m_addUsername; }
 	const char* GetAddPassword() { return m_addPassword; }
 	int GetControlPort() { return m_controlPort; }
+	bool GetFormAuth() { return m_formAuth; }
 	bool GetSecureControl() { return m_secureControl; }
 	int GetSecurePort() { return m_securePort; }
 	const char* GetSecureCert() { return m_secureCert; }
 	const char* GetSecureKey() { return m_secureKey; }
+	const char* GetCertStore() { return m_certStore; }
+	bool GetCertCheck() { return m_certCheck; }
 	const char* GetAuthorizedIp() { return m_authorizedIp; }
 	const char* GetLockFile() { return m_lockFile; }
 	const char* GetDaemonUsername() { return m_daemonUsername; }
@@ -231,15 +253,14 @@ public:
 	bool GetParRepair() { return m_parRepair; }
 	EParScan GetParScan() { return m_parScan; }
 	bool GetParQuick() { return m_parQuick; }
+	EPostStrategy GetPostStrategy() { return m_postStrategy; }
 	bool GetParRename() { return m_parRename; }
 	int GetParBuffer() { return m_parBuffer; }
 	int GetParThreads() { return m_parThreads; }
+	bool GetRarRename() { return m_rarRename; }
 	EHealthCheck GetHealthCheck() { return m_healthCheck; }
 	const char* GetScriptOrder() { return m_scriptOrder; }
-	const char* GetPostScript() { return m_postScript; }
-	const char* GetScanScript() { return m_scanScript; }
-	const char* GetQueueScript() { return m_queueScript; }
-	const char* GetFeedScript() { return m_feedScript; }
+	const char* GetExtensions() { return m_extensions; }
 	int GetUMask() { return m_umask; }
 	int GetUpdateInterval() {return m_updateInterval; }
 	bool GetCursesNzbName() { return m_cursesNzbName; }
@@ -252,7 +273,8 @@ public:
 	int GetNzbDirFileAge() { return m_nzbDirFileAge; }
 	int GetDiskSpace() { return m_diskSpace; }
 	bool GetTls() { return m_tls; }
-	bool GetDumpCore() { return m_dumpCore; }
+	bool GetCrashTrace() { return m_crashTrace; }
+	bool GetCrashDump() { return m_crashDump; }
 	bool GetParPauseQueue() { return m_parPauseQueue; }
 	bool GetScriptPauseQueue() { return m_scriptPauseQueue; }
 	bool GetNzbCleanupDisk() { return m_nzbCleanupDisk; }
@@ -260,6 +282,7 @@ public:
 	int GetKeepHistory() { return m_keepHistory; }
 	bool GetAccurateRate() { return m_accurateRate; }
 	bool GetUnpack() { return m_unpack; }
+	bool GetDirectUnpack() { return m_directUnpack; }
 	bool GetUnpackCleanupDisk() { return m_unpackCleanupDisk; }
 	const char* GetUnrarCmd() { return m_unrarCmd; }
 	const char* GetSevenZipCmd() { return m_sevenZipCmd; }
@@ -267,6 +290,7 @@ public:
 	bool GetUnpackPauseQueue() { return m_unpackPauseQueue; }
 	const char* GetExtCleanupDisk() { return m_extCleanupDisk; }
 	const char* GetParIgnoreExt() { return m_parIgnoreExt; }
+	const char* GetUnpackIgnoreExt() { return m_unpackIgnoreExt; }
 	int GetFeedHistory() { return m_feedHistory; }
 	bool GetUrlForce() { return m_urlForce; }
 	int GetTimeCorrection() { return m_timeCorrection; }
@@ -277,6 +301,9 @@ public:
 	int GetMonthlyQuota() { return m_monthlyQuota; }
 	int GetQuotaStartDay() { return m_quotaStartDay; }
 	int GetDailyQuota() { return m_dailyQuota; }
+	bool GetDirectRename() { return m_directRename; }
+	bool GetReorderFiles() { return m_reorderFiles; }
+	EFileNaming GetFileNaming() { return m_fileNaming; }
 
 	Categories* GetCategories() { return &m_categories; }
 	Category* FindCategory(const char* name, bool searchAliases) { return m_categories.FindCategory(name, searchAliases); }
@@ -341,10 +368,13 @@ private:
 	int m_articleTimeout = 0;
 	int m_urlTimeout = 0;
 	int m_terminateTimeout = 0;
+	int m_remoteTimeout = 0;
 	bool m_appendCategoryDir = false;
 	bool m_continuePartial = false;
-	int m_retries = 0;
-	int m_retryInterval = 0;
+	int m_articleRetries = 0;
+	int m_articleInterval = 0;
+	int m_urlRetries = 0;
+	int m_urlInterval = 0;
 	bool m_saveQueue = false;
 	bool m_flushQueue = false;
 	bool m_dupeCheck = false;
@@ -355,11 +385,14 @@ private:
 	CString m_restrictedPassword;
 	CString m_addUsername;
 	CString m_addPassword;
+	bool m_formAuth = false;
 	int m_controlPort = 0;
 	bool m_secureControl = false;
 	int m_securePort = 0;
 	CString m_secureCert;
 	CString m_secureKey;
+	CString m_certStore;
+	bool m_certCheck = false;
 	CString m_authorizedIp;
 	CString m_lockFile;
 	CString m_daemonUsername;
@@ -374,15 +407,15 @@ private:
 	bool m_parRepair = false;
 	EParScan m_parScan = psLimited;
 	bool m_parQuick = true;
+	EPostStrategy m_postStrategy = ppSequential;
 	bool m_parRename = false;
 	int m_parBuffer = 0;
 	int m_parThreads = 0;
+	bool m_rarRename = false;
+	bool m_directRename = false;
 	EHealthCheck m_healthCheck = hcNone;
-	CString m_postScript;
+	CString m_extensions;
 	CString m_scriptOrder;
-	CString m_scanScript;
-	CString m_queueScript;
-	CString m_feedScript;
 	int m_umask = 0;
 	int m_updateInterval = 0;
 	bool m_cursesNzbName = false;
@@ -395,7 +428,8 @@ private:
 	int m_nzbDirFileAge = 0;
 	int m_diskSpace = 0;
 	bool m_tls = false;
-	bool m_dumpCore = false;
+	bool m_crashTrace = false;
+	bool m_crashDump = false;
 	bool m_parPauseQueue = false;
 	bool m_scriptPauseQueue = false;
 	bool m_nzbCleanupDisk = false;
@@ -403,6 +437,7 @@ private:
 	int m_keepHistory = 0;
 	bool m_accurateRate = false;
 	bool m_unpack = false;
+	bool m_directUnpack = false;
 	bool m_unpackCleanupDisk = false;
 	CString m_unrarCmd;
 	CString m_sevenZipCmd;
@@ -410,6 +445,7 @@ private:
 	bool m_unpackPauseQueue;
 	CString m_extCleanupDisk;
 	CString m_parIgnoreExt;
+	CString m_unpackIgnoreExt;
 	int m_feedHistory = 0;
 	bool m_urlForce = false;
 	int m_timeCorrection = 0;
@@ -420,6 +456,8 @@ private:
 	int m_monthlyQuota = 0;
 	int m_quotaStartDay = 0;
 	int m_dailyQuota = 0;
+	bool m_reorderFiles = false;
+	EFileNaming m_fileNaming = nfArticle;
 
 	// Current state
 	bool m_serverMode = false;
@@ -461,7 +499,9 @@ private:
 	void ConfigError(const char* msg, ...);
 	void ConfigWarn(const char* msg, ...);
 	void LocateOptionSrcPos(const char *optionName);
-	void ConvertOldOption(CString& option, CString& value);
+	static void ConvertOldOption(CString& option, CString& value);
+	static void MergeOldScriptOption(OptEntries* optEntries, const char* optname, bool mergeCategories);
+	static bool HasScript(const char* scriptList, const char* scriptName);
 };
 
 extern Options* g_Options;
