@@ -58,9 +58,10 @@ void QueueCoordinator::CoordinatorDownloadQueue::Save()
 		return;
 	}
 
-	if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+	if (g_Options->GetServerMode())
 	{
 		g_DiskState->SaveDownloadQueue(this, m_historyChanged);
+		m_stateChanged = true;
 	}
 
 	for (NzbInfo* nzbInfo : GetQueue())
@@ -102,11 +103,11 @@ void QueueCoordinator::Load()
 	bool perfectServerMatch = true;
 	bool queueLoaded = false;
 
-	if (g_Options->GetServerMode() && g_Options->GetSaveQueue())
+	if (g_Options->GetServerMode())
 	{
 		statLoaded = g_StatMeter->Load(&perfectServerMatch);
 
-		if (g_Options->GetReloadQueue() && g_DiskState->DownloadQueueExists())
+		if (g_DiskState->DownloadQueueExists())
 		{
 			queueLoaded = g_DiskState->LoadDownloadQueue(downloadQueue, g_ServerPool->GetServers());
 		}
@@ -133,7 +134,7 @@ void QueueCoordinator::Load()
 		downloadQueue->Save();
 
 		// re-save file states into diskstate to update server ids
-		if (g_Options->GetServerMode() && g_Options->GetSaveQueue())
+		if (g_Options->GetServerMode())
 		{
 			for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
 			{
@@ -256,12 +257,14 @@ void QueueCoordinator::Run()
 			}
 			resetCounter = 0;
 			g_StatMeter->IntervalCheck();
+			g_Log->IntervalCheck();
 			AdjustDownloadsLimit();
 		}
 	}
 
 	WaitJobs();
 	SaveAllPartialState();
+	SaveAllFileState();
 
 	debug("Exiting QueueCoordinator-loop");
 }
@@ -331,7 +334,7 @@ NzbInfo* QueueCoordinator::AddNzbFileToQueue(std::unique_ptr<NzbInfo> nzbInfo, N
 		for (FileInfo* fileInfo: nzbInfo->GetFileList())
 		{
 			allPaused &= fileInfo->GetPaused();
-			if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+			if (g_Options->GetServerMode())
 			{
 				g_DiskState->DiscardFile(fileInfo->GetId(), true, false, false);
 			}
@@ -398,11 +401,6 @@ void QueueCoordinator::CheckDupeFileInfos(NzbInfo* nzbInfo)
 {
 	debug("CheckDupeFileInfos");
 
-	if (!g_Options->GetDupeCheck() || nzbInfo->GetDupeMode() == dmForce)
-	{
-		return;
-	}
-
 	RawFileList dupeList;
 
 	int index1 = 0;
@@ -419,9 +417,19 @@ void QueueCoordinator::CheckDupeFileInfos(NzbInfo* nzbInfo)
 				(fileInfo->GetSize() < fileInfo2->GetSize() ||
 				 (fileInfo->GetSize() == fileInfo2->GetSize() && index2 < index1)))
 			{
-				warn("File \"%s\" appears twice in collection, adding only the biggest file", fileInfo->GetFilename());
-				dupe = true;
-				break;
+				// If more than two files have same filename we don't filter them out since that
+				// naming might be intentional and correct filenames must be read from article bodies.
+				int dupeCount = std::count_if(nzbInfo->GetFileList()->begin(), nzbInfo->GetFileList()->end(),
+					[fileInfo2](std::unique_ptr<FileInfo>& fileInfo3)
+					{
+						return !strcmp(fileInfo3->GetFilename(), fileInfo2->GetFilename());
+					});
+				if (dupeCount == 2)
+				{
+					warn("File \"%s\" appears twice in collection, adding only the biggest file", fileInfo->GetFilename());
+					dupe = true;
+					break;
+				}
 			}
 		}
 		if (dupe)
@@ -435,7 +443,7 @@ void QueueCoordinator::CheckDupeFileInfos(NzbInfo* nzbInfo)
 	{
 		nzbInfo->UpdateDeletedStats(fileInfo);
 		nzbInfo->GetFileList()->Remove(fileInfo);
-		if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+		if (g_Options->GetServerMode())
 		{
 			g_DiskState->DiscardFile(fileInfo->GetId(), true, false, false);
 		}
@@ -464,8 +472,7 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 	// if the file doesn't have any articles left for download, we store that fact and search again,
 	// ignoring all files which were previously marked as not having any articles.
 
-	// special case: if the file has ExtraPriority-flag set, it has the highest priority and the
-	// Paused-flag is ignored.
+	// special case: if the file has ExtraPriority-flag set, it has the highest priority.
 
 	//debug("QueueCoordinator::GetNextArticle()");
 
@@ -480,20 +487,34 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 
 		for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
 		{
-			for (FileInfo* fileInfo1 : nzbInfo->GetFileList())
+			bool nzbHigherPriority = fileInfo &&
+				((nzbInfo->HasExtraPriority() == fileInfo->GetNzbInfo()->HasExtraPriority() &&
+					nzbInfo->GetPriority() > fileInfo->GetNzbInfo()->GetPriority()) ||
+					(nzbInfo->HasExtraPriority() > fileInfo->GetNzbInfo()->HasExtraPriority()));
+
+			bool nzbPaused = nzbInfo->GetFileList()->size() - nzbInfo->GetPausedFileCount() <= 0;
+
+			if ((!fileInfo || nzbHigherPriority) && !nzbPaused &&
+				(!(g_Options->GetPauseDownload() || g_Options->GetQuotaReached()) || nzbInfo->GetForcePriority()))
 			{
-				if ((checkedFiles.empty() ||
-					 std::find(checkedFiles.begin(), checkedFiles.end(), fileInfo1) == checkedFiles.end()) &&
-					!fileInfo1->GetPaused() && !fileInfo1->GetDeleted() &&
-					(g_Options->GetPropagationDelay() == 0 ||
-					 (int)fileInfo1->GetTime() < (int)curDate - g_Options->GetPropagationDelay()) &&
-					(!(g_Options->GetPauseDownload() || g_Options->GetQuotaReached()) || nzbInfo->GetForcePriority()) &&
-					(!fileInfo ||
-					 (fileInfo1->GetExtraPriority() == fileInfo->GetExtraPriority() &&
-					  fileInfo1->GetNzbInfo()->GetPriority() > fileInfo->GetNzbInfo()->GetPriority()) ||
-					 (fileInfo1->GetExtraPriority() > fileInfo->GetExtraPriority())))
+				for (FileInfo* fileInfo1 : nzbInfo->GetFileList())
 				{
-					fileInfo = fileInfo1;
+					bool alreadyChecked = !checkedFiles.empty() &&
+						std::find(checkedFiles.begin(), checkedFiles.end(), fileInfo1) != checkedFiles.end();
+
+					bool propagationWait = g_Options->GetPropagationDelay() > 0 &&
+						(int)fileInfo1->GetTime() + g_Options->GetPropagationDelay() >= (int)curDate;
+
+					bool higherPriority = fileInfo &&
+						((fileInfo1->GetExtraPriority() == fileInfo->GetExtraPriority() &&
+							fileInfo1->GetNzbInfo()->GetPriority() > fileInfo->GetNzbInfo()->GetPriority()) ||
+							(fileInfo1->GetExtraPriority() > fileInfo->GetExtraPriority()));
+
+					if (!alreadyChecked && !propagationWait && !fileInfo1->GetPaused() && 
+						!fileInfo1->GetDeleted() && (!fileInfo || higherPriority))
+					{
+						fileInfo = fileInfo1;
+					}
 				}
 			}
 		}
@@ -512,7 +533,7 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 			return true;
 		}
 
-		if (fileInfo->GetArticles()->empty() && g_Options->GetSaveQueue() && g_Options->GetServerMode())
+		if (fileInfo->GetArticles()->empty() && g_Options->GetServerMode())
 		{
 			g_DiskState->LoadArticles(fileInfo);
 			LoadPartialState(fileInfo);
@@ -546,7 +567,7 @@ bool QueueCoordinator::GetNextFirstArticle(NzbInfo* nzbInfo, FileInfo* &fileInfo
 	{
 		if (!fileInfo1->GetFilenameConfirmed())
 		{
-			if (fileInfo1->GetArticles()->empty() && g_Options->GetSaveQueue() && g_Options->GetServerMode())
+			if (fileInfo1->GetArticles()->empty() && g_Options->GetServerMode())
 			{
 				g_DiskState->LoadArticles(fileInfo1);
 				LoadPartialState(fileInfo1);
@@ -582,7 +603,7 @@ void QueueCoordinator::StartArticleDownload(FileInfo* fileInfo, ArticleInfo* art
 	articleDownloader->SetArticleInfo(articleInfo);
 	articleDownloader->SetConnection(connection);
 
-	if (articleInfo->GetPartNumber() == 1 && g_Options->GetDirectRename() && g_Options->GetDecode())
+	if (articleInfo->GetPartNumber() == 1 && g_Options->GetDirectRename() && !g_Options->GetRawArticle())
 	{
 		articleDownloader->SetContentAnalyzer(m_directRenamer.MakeArticleContentAnalyzer());
 	}
@@ -779,7 +800,7 @@ void QueueCoordinator::DeleteFileInfo(DownloadQueue* downloadQueue, FileInfo* fi
 		fileInfo->GetSuccessArticles() > 0 || fileInfo->GetFailedArticles() > 0 ? CompletedFile::cfPartial :
 		CompletedFile::cfNone;
 
-	if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+	if (g_Options->GetServerMode())
 	{
 		g_DiskState->DiscardFile(fileInfo->GetId(), fileStatus == CompletedFile::cfSuccess || (fileDeleted && !parking), true, false);
 		if (fileStatus == CompletedFile::cfPartial && (completed || parking))
@@ -847,7 +868,7 @@ void QueueCoordinator::DiscardTempFiles(FileInfo* fileInfo)
 
 void QueueCoordinator::SaveAllPartialState()
 {
-	if (!(g_Options->GetServerMode() && g_Options->GetSaveQueue()))
+	if (!g_Options->GetServerMode())
 	{
 		return;
 	}
@@ -906,6 +927,15 @@ void QueueCoordinator::LoadPartialState(FileInfo* fileInfo)
 	}
 }
 
+void QueueCoordinator::SaveAllFileState()
+{
+	if (g_Options->GetServerMode() && m_downloadQueue.m_stateChanged)
+	{
+		GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
+		g_DiskState->SaveAllFileInfos(downloadQueue);
+	}
+}
+
 void QueueCoordinator::CheckHealth(DownloadQueue* downloadQueue, FileInfo* fileInfo)
 {
 	if (g_Options->GetHealthCheck() == Options::hcNone ||
@@ -962,7 +992,7 @@ void QueueCoordinator::LogDebugInfo()
 
 void QueueCoordinator::ResetHangingDownloads()
 {
-	if (g_Options->GetTerminateTimeout() == 0 && g_Options->GetArticleTimeout() == 0)
+	if (g_Options->GetArticleTimeout() == 0)
 	{
 		return;
 	}
@@ -970,46 +1000,16 @@ void QueueCoordinator::ResetHangingDownloads()
 	GuardedDownloadQueue guard = DownloadQueue::Guard();
 	time_t tm = Util::CurrentTime();
 
-	m_activeDownloads.erase(std::remove_if(m_activeDownloads.begin(), m_activeDownloads.end(),
-		[tm](ArticleDownloader* articleDownloader)
+	for (ArticleDownloader* articleDownloader : m_activeDownloads)
+	{
+		if (tm - articleDownloader->GetLastUpdateTime() > g_Options->GetArticleTimeout() + 1 &&
+			articleDownloader->GetStatus() == ArticleDownloader::adRunning)
 		{
-			if (tm - articleDownloader->GetLastUpdateTime() > g_Options->GetArticleTimeout() + 1 &&
-				articleDownloader->GetStatus() == ArticleDownloader::adRunning)
-			{
-				error("Cancelling hanging download %s @ %s", articleDownloader->GetInfoName(),
-					articleDownloader->GetConnectionName());
-				articleDownloader->Stop();
-			}
-
-			if (tm - articleDownloader->GetLastUpdateTime() > g_Options->GetTerminateTimeout() &&
-				articleDownloader->GetStatus() == ArticleDownloader::adRunning)
-			{
-				ArticleInfo* articleInfo = articleDownloader->GetArticleInfo();
-				debug("Terminating hanging download %s", articleDownloader->GetInfoName());
-				if (articleDownloader->Terminate())
-				{
-					error("Terminated hanging download %s @ %s", articleDownloader->GetInfoName(),
-						articleDownloader->GetConnectionName());
-					articleInfo->SetStatus(ArticleInfo::aiUndefined);
-				}
-				else
-				{
-					error("Could not terminate hanging download %s @ %s", articleDownloader->GetInfoName(),
-						articleDownloader->GetConnectionName());
-				}
-
-				articleDownloader->GetFileInfo()->SetActiveDownloads(articleDownloader->GetFileInfo()->GetActiveDownloads() - 1);
-				articleDownloader->GetFileInfo()->GetNzbInfo()->SetActiveDownloads(articleDownloader->GetFileInfo()->GetNzbInfo()->GetActiveDownloads() - 1);
-				articleDownloader->GetFileInfo()->GetNzbInfo()->SetDownloadedSize(articleDownloader->GetFileInfo()->GetNzbInfo()->GetDownloadedSize() + articleDownloader->GetDownloadedSize());
-
-				// it's not safe to destroy pArticleDownloader, because the state of object is unknown
-				delete articleDownloader;
-
-				return true;
-			}
-			return false;
-		}),
-		m_activeDownloads.end());
+			error("Cancelling hanging download %s @ %s", articleDownloader->GetInfoName(),
+				articleDownloader->GetConnectionName());
+			articleDownloader->Stop();
+		}
+	}
 }
 
 /*
@@ -1292,7 +1292,7 @@ void QueueCoordinator::DirectRenameCompleted(DownloadQueue* downloadQueue, NzbIn
 {
 	for (FileInfo* fileInfo : nzbInfo->GetFileList())
 	{
-		if (g_Options->GetSaveQueue() && g_Options->GetServerMode() && !fileInfo->GetArticles()->empty())
+		if (g_Options->GetServerMode() && !fileInfo->GetArticles()->empty())
 		{
 			// save new file name into disk state file
 			g_DiskState->SaveFile(fileInfo);
@@ -1328,55 +1328,36 @@ void QueueCoordinator::DiscardDirectRename(DownloadQueue* downloadQueue, NzbInfo
 
 	for (FileInfo* fileInfo : nzbInfo->GetFileList())
 	{
-		if (fileInfo->GetParFile() && fileInfo->GetCompletedArticles() == 1 && fileInfo->GetActiveDownloads() == 0)
+		if (fileInfo->GetParFile() && fileInfo->GetCompletedArticles() == 1 &&
+			fileInfo->GetActiveDownloads() == 0)
 		{
-			// discard downloaded articles from partially downloaded par-files
-			discardedSize += fileInfo->GetSuccessSize();
-			discardedCount++;
-
-			nzbInfo->SetCurrentSuccessArticles(nzbInfo->GetCurrentSuccessArticles() - fileInfo->GetSuccessArticles());
-			nzbInfo->SetCurrentSuccessSize(nzbInfo->GetCurrentSuccessSize() - fileInfo->GetSuccessSize());
-			nzbInfo->SetParCurrentSuccessSize(nzbInfo->GetParCurrentSuccessSize() - fileInfo->GetSuccessSize());
-			fileInfo->SetSuccessSize(0);
-			fileInfo->SetSuccessArticles(0);
-
-			nzbInfo->SetCurrentFailedArticles(nzbInfo->GetCurrentFailedArticles() - fileInfo->GetFailedArticles());
-			nzbInfo->SetCurrentFailedSize(nzbInfo->GetCurrentFailedSize() - fileInfo->GetFailedSize());
-			nzbInfo->SetParCurrentFailedSize(nzbInfo->GetParCurrentFailedSize() - fileInfo->GetFailedSize());
-			fileInfo->SetFailedSize(0);
-			fileInfo->SetFailedArticles(0);
-
-			fileInfo->SetCompletedArticles(0);
-			fileInfo->SetRemainingSize(fileInfo->GetSize() - fileInfo->GetMissedSize());
-
-			// discard temporary files
-			DiscardTempFiles(fileInfo);
-			g_DiskState->DiscardFile(fileInfo->GetId(), false, true, false);
-
-			fileInfo->SetOutputFilename(nullptr);
-			fileInfo->SetOutputInitialized(false);
-			fileInfo->SetCachedArticles(0);
-			fileInfo->SetPartialChanged(false);
-			fileInfo->SetPartialState(FileInfo::psNone);
-
-			if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+			bool locked = false;
 			{
-				// free up memory used by articles if possible
-				fileInfo->GetArticles()->clear();
-			}
-			else
-			{
-				// reset article states if discarding isn't possible
-				for (ArticleInfo* articleInfo : fileInfo->GetArticles())
+				Guard contentGuard = g_ArticleCache->GuardContent();
+				locked = fileInfo->GetFlushLocked();
+				if (!locked)
 				{
-					articleInfo->SetStatus(ArticleInfo::aiUndefined);
-					articleInfo->SetResultFilename(nullptr);
-					articleInfo->DiscardSegment();
+					fileInfo->SetFlushLocked(true);
 				}
 			}
+
+			if (!locked)
+			{
+				// discard downloaded articles from partially downloaded par-files
+				discardedSize += fileInfo->GetSuccessSize();
+				discardedCount++;
+
+				DiscardDownloadedArticles(nzbInfo, fileInfo);
+			}
+
+			if (!locked)
+			{
+				Guard contentGuard = g_ArticleCache->GuardContent();
+				fileInfo->SetFlushLocked(false);
+			}
 		}
 
-		if (g_Options->GetSaveQueue() && g_Options->GetServerMode() &&
+		if (g_Options->GetServerMode() &&
 			!fileInfo->GetArticles()->empty() && g_Options->GetContinuePartial() &&
 			fileInfo->GetActiveDownloads() == 0 && fileInfo->GetCachedArticles() == 0)
 		{
@@ -1394,3 +1375,55 @@ void QueueCoordinator::DiscardDirectRename(DownloadQueue* downloadQueue, NzbInfo
 			*Util::FormatSize(discardedSize), discardedCount);
 	}
 }
+
+void QueueCoordinator::DiscardDownloadedArticles(NzbInfo* nzbInfo, FileInfo* fileInfo)
+{
+	nzbInfo->SetRemainingSize(nzbInfo->GetRemainingSize() + fileInfo->GetSuccessSize() + fileInfo->GetFailedSize());
+	if (fileInfo->GetPaused())
+	{
+		nzbInfo->SetPausedSize(nzbInfo->GetPausedSize() + fileInfo->GetSuccessSize() + fileInfo->GetFailedSize());
+	}
+	nzbInfo->GetCurrentServerStats()->ListOp(fileInfo->GetServerStats(), ServerStatList::soSubtract);
+	fileInfo->GetServerStats()->clear();
+
+	nzbInfo->SetCurrentSuccessArticles(nzbInfo->GetCurrentSuccessArticles() - fileInfo->GetSuccessArticles());
+	nzbInfo->SetCurrentSuccessSize(nzbInfo->GetCurrentSuccessSize() - fileInfo->GetSuccessSize());
+	nzbInfo->SetParCurrentSuccessSize(nzbInfo->GetParCurrentSuccessSize() - fileInfo->GetSuccessSize());
+	fileInfo->SetSuccessSize(0);
+	fileInfo->SetSuccessArticles(0);
+
+	nzbInfo->SetCurrentFailedArticles(nzbInfo->GetCurrentFailedArticles() - fileInfo->GetFailedArticles());
+	nzbInfo->SetCurrentFailedSize(nzbInfo->GetCurrentFailedSize() - fileInfo->GetFailedSize());
+	nzbInfo->SetParCurrentFailedSize(nzbInfo->GetParCurrentFailedSize() - fileInfo->GetFailedSize());
+	fileInfo->SetFailedSize(0);
+	fileInfo->SetFailedArticles(0);
+
+	fileInfo->SetCompletedArticles(0);
+	fileInfo->SetRemainingSize(fileInfo->GetSize() - fileInfo->GetMissedSize());
+
+	// discard temporary files
+	DiscardTempFiles(fileInfo);
+	g_DiskState->DiscardFile(fileInfo->GetId(), false, true, false);
+
+	fileInfo->SetOutputFilename(nullptr);
+	fileInfo->SetOutputInitialized(false);
+	fileInfo->SetCachedArticles(0);
+	fileInfo->SetPartialChanged(false);
+	fileInfo->SetPartialState(FileInfo::psNone);
+
+	if (g_Options->GetServerMode())
+	{
+		// free up memory used by articles if possible
+		fileInfo->GetArticles()->clear();
+	}
+	else
+	{
+		// reset article states if discarding isn't possible
+		for (ArticleInfo* articleInfo : fileInfo->GetArticles())
+		{
+			articleInfo->SetStatus(ArticleInfo::aiUndefined);
+			articleInfo->SetResultFilename(nullptr);
+			articleInfo->DiscardSegment();
+		}
+	}
+}
