@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2012-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,13 @@
 #include "Util.h"
 #include "FileSystem.h"
 
+#ifndef DISABLE_PARCHECK
+#include "par2cmdline.h"
+#include "md5.h"
+#endif
+
+static const char* ERR_HTTP_OK = "200 OK";
+static const char* ERR_HTTP_NOT_MODIFIED = "304 Not Modified";
 static const char* ERR_HTTP_BAD_REQUEST = "400 Bad Request";
 static const char* ERR_HTTP_NOT_FOUND = "404 Not Found";
 static const char* ERR_HTTP_SERVICE_UNAVAILABLE = "503 Service Unavailable";
@@ -174,6 +181,14 @@ void WebProcessor::ParseHeaders()
 		{
 			m_forwardedFor = p + 17;
 		}
+		else if (!strncasecmp(p, "If-None-Match: ", 15))
+		{
+			m_oldETag = p + 15;
+		}
+		else if (!strncasecmp(p, "Connection: keep-alive", 22))
+		{
+			m_keepAlive = true;
+		}
 		else if (*p == '\0')
 		{
 			break;
@@ -314,7 +329,7 @@ void WebProcessor::Dispatch()
 		processor.SetUserAccess((XmlRpcProcessor::EUserAccess)m_userAccess);
 		processor.SetUrl(m_url);
 		processor.Execute();
-		SendBodyResponse(processor.GetResponse(), strlen(processor.GetResponse()), processor.GetContentType());
+		SendBodyResponse(processor.GetResponse(), strlen(processor.GetResponse()), processor.GetContentType(), processor.IsSafeMethod());
 		return;
 	}
 
@@ -330,43 +345,42 @@ void WebProcessor::Dispatch()
 		return;
 	}
 
-	// for security reasons we allow only characters "0..9 A..Z a..z . - _ /" in the URLs
+	// for security reasons we allow only characters "0..9 A..Z a..z . - + _ / ?" in the URLs
 	// we also don't allow ".." in the URLs
 	for (char *p = m_url; *p; p++)
 	{
 		if (!((*p >= '0' && *p <= '9') || (*p >= 'A' && *p <= 'Z') || (*p >= 'a' && *p <= 'z') ||
-			*p == '.' || *p == '-' || *p == '_' || *p == '/') || (*p == '.' && p[1] == '.'))
+			*p == '.' || *p == '-' || *p == '+' || *p == '?' || *p == '_' || *p == '/') || (*p == '.' && p[1] == '.'))
 		{
 			SendErrorResponse(ERR_HTTP_NOT_FOUND, true);
 			return;
 		}
 	}
 
-	const char *defRes = "";
-	if (m_url[strlen(m_url)-1] == '/')
+	if (!strncmp(m_url, "/combined.", 10) && strchr(m_url, '?'))
 	{
-		// default file in directory (if not specified) is "index.html"
-		defRes = "index.html";
+		SendMultiFileResponse();
+	}
+	else
+	{
+		SendSingleFileResponse();
 	}
-
-	BString<1024> disk_filename("%s%s%s", g_Options->GetWebDir(), *m_url, defRes);
-
-	SendFileResponse(disk_filename);
 }
 
 void WebProcessor::SendAuthResponse()
 {
 	const char* AUTH_RESPONSE_HEADER =
-		"HTTP/1.0 401 Unauthorized\r\n"
+		"HTTP/1.1 401 Unauthorized\r\n"
 		"%s"
-		"Connection: close\r\n"
+		"Connection: %s\r\n"
 		"Content-Type: text/plain\r\n"
+		"Content-Length: 0\r\n"
 		"Server: nzbget-%s\r\n"
 		"\r\n";
 
 	BString<1024> responseHeader(AUTH_RESPONSE_HEADER,
 		g_Options->GetFormAuth() ? "" : "WWW-Authenticate: Basic realm=\"NZBGet\"\r\n",
-		Util::VersionRevision());
+		m_keepAlive ? "keep-alive" : "close", Util::VersionRevision());
 
 	// Send the response answer
 	debug("ResponseHeader=%s", *responseHeader);
@@ -377,8 +391,9 @@ void WebProcessor::SendOptionsResponse()
 {
 	const char* OPTIONS_RESPONSE_HEADER =
 		"HTTP/1.1 200 OK\r\n"
-		"Connection: close\r\n"
+		"Connection: %s\r\n"
 		//"Content-Type: plain/text\r\n"
+		"Content-Length: 0\r\n"
 		"Access-Control-Allow-Methods: GET, POST, OPTIONS\r\n"
 		"Access-Control-Allow-Origin: %s\r\n"
 		"Access-Control-Allow-Credentials: true\r\n"
@@ -387,6 +402,7 @@ void WebProcessor::SendOptionsResponse()
 		"Server: nzbget-%s\r\n"
 		"\r\n";
 	BString<1024> responseHeader(OPTIONS_RESPONSE_HEADER,
+		m_keepAlive ? "keep-alive" : "close", 
 		m_origin.Str(), Util::VersionRevision());
 
 	// Send the response answer
@@ -397,8 +413,8 @@ void WebProcessor::SendOptionsResponse()
 void WebProcessor::SendErrorResponse(const char* errCode, bool printWarning)
 {
 	const char* RESPONSE_HEADER =
-		"HTTP/1.0 %s\r\n"
-		"Connection: close\r\n"
+		"HTTP/1.1 %s\r\n"
+		"Connection: %s\r\n"
 		"Content-Length: %i\r\n"
 		"Content-Type: text/html\r\n"
 		"Server: nzbget-%s\r\n"
@@ -413,7 +429,9 @@ void WebProcessor::SendErrorResponse(const char* errCode, bool printWarning)
 		errCode, errCode);
 	int pageContentLen = responseBody.Length();
 
-	BString<1024> responseHeader(RESPONSE_HEADER, errCode, pageContentLen, Util::VersionRevision());
+	BString<1024> responseHeader(RESPONSE_HEADER, errCode,
+		m_keepAlive ? "keep-alive" : "close",
+		pageContentLen, Util::VersionRevision());
 
 	// Send the response answer
 	m_connection->Send(responseHeader, responseHeader.Length());
@@ -423,23 +441,25 @@ void WebProcessor::SendErrorResponse(const char* errCode, bool printWarning)
 void WebProcessor::SendRedirectResponse(const char* url)
 {
 	const char* REDIRECT_RESPONSE_HEADER =
-		"HTTP/1.0 301 Moved Permanently\r\n"
+		"HTTP/1.1 301 Moved Permanently\r\n"
 		"Location: %s\r\n"
-		"Connection: close\r\n"
+		"Connection: %s\r\n"
+		"Content-Length: 0\r\n"
 		"Server: nzbget-%s\r\n"
 		"\r\n";
-	BString<1024> responseHeader(REDIRECT_RESPONSE_HEADER, url, Util::VersionRevision());
+	BString<1024> responseHeader(REDIRECT_RESPONSE_HEADER, url,
+		m_keepAlive ? "keep-alive" : "close", Util::VersionRevision());
 
 	// Send the response answer
 	debug("ResponseHeader=%s", *responseHeader);
 	m_connection->Send(responseHeader, responseHeader.Length());
 }
 
-void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* contentType)
+void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* contentType, bool cachable)
 {
 	const char* RESPONSE_HEADER =
-		"HTTP/1.1 200 OK\r\n"
-		"Connection: close\r\n"
+		"HTTP/1.1 %s\r\n"
+		"Connection: %s\r\n"
 		"Access-Control-Allow-Methods: GET, POST, OPTIONS\r\n"
 		"Access-Control-Allow-Origin: %s\r\n"
 		"Access-Control-Allow-Credentials: true\r\n"
@@ -450,9 +470,37 @@ void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* c
 		"Content-Length: %i\r\n"
 		"%s"					// Content-Type: xxx
 		"%s"					// Content-Encoding: gzip
+		"%s"					// ETag
 		"Server: nzbget-%s\r\n"
 		"\r\n";
 
+	BString<1024> eTagHeader;
+	bool unchanged = false;
+
+	if (cachable)
+	{
+		BString<1024> newETag;
+
+#ifndef DISABLE_PARCHECK
+		Par2::MD5Hash hash;
+		Par2::MD5Context md5;
+		md5.Update(body, bodyLen);
+		md5.Final(hash);
+		newETag.Format("\"%s\"", hash.print().c_str());
+#else
+		uint32 hash = Util::HashBJ96(body, bodyLen, 0);
+		newETag.Format("\"%x\"", hash);
+#endif
+
+		unchanged = m_oldETag && !strcmp(newETag, m_oldETag);
+		if (unchanged)
+		{
+			body = "";
+			bodyLen = 0;
+		}
+		eTagHeader.Format("ETag: %s\r\n", *newETag);
+	}
+
 #ifndef DISABLE_GZIP
 	CharBuffer gbuf;
 	bool gzip = m_gzip && bodyLen > MAX_UNCOMPRESSED_SIZE;
@@ -482,24 +530,39 @@ void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* c
 	}
 
 	BString<1024> responseHeader(RESPONSE_HEADER,
+		unchanged ? ERR_HTTP_NOT_MODIFIED : ERR_HTTP_OK,
+		m_keepAlive ? "keep-alive" : "close",
 		m_origin.Str(),
 		g_Options->GetFormAuth() ? "form" : "http",
 		m_authorized ? m_serverAuthToken[m_userAccess] : "",
-		bodyLen, *contentTypeHeader,
+		bodyLen,
+		*contentTypeHeader,
 		gzip ? "Content-Encoding: gzip\r\n" : "",
+		cachable ? *eTagHeader : "",
 		Util::VersionRevision());
 
+	debug("[%s] (%s) %s", *m_url, *m_oldETag, *responseHeader);
+
 	// Send the request answer
 	m_connection->Send(responseHeader, responseHeader.Length());
 	m_connection->Send(body, bodyLen);
 }
 
-void WebProcessor::SendFileResponse(const char* filename)
+void WebProcessor::SendSingleFileResponse()
 {
-	debug("serving file: %s", filename);
+	const char *defRes = "";
+	if (m_url[strlen(m_url)-1] == '/')
+	{
+		// default file in directory (if not specified) is "index.html"
+		defRes = "index.html";
+	}
+
+	BString<1024> filename("%s%s%s", g_Options->GetWebDir(), *m_url, defRes);
+
+	debug("serving file: %s", *filename);
 
 	CharBuffer body;
-	if (!FileSystem::LoadFileIntoBuffer(filename, body, false))
+	if (!FileSystem::LoadFileIntoBuffer(filename, body, true))
 	{
 		// do not print warnings "404 not found" for certain files
 		bool ignorable = !strcmp(filename, "package-info.json") ||
@@ -510,7 +573,48 @@ void WebProcessor::SendFileResponse(const char* filename)
 		return;
 	}
 
-	SendBodyResponse(body, body.Size(), DetectContentType(filename));
+	const char* contentType = DetectContentType(filename);
+	int len = body.Size() - 1;
+
+#ifdef DEBUG
+	if (contentType && !strcmp(contentType, "text/html"))
+	{
+		Util::ReduceStr(body, "<!-- %if-debug%", "");
+		Util::ReduceStr(body, "<!-- %if-not-debug% -->", "<!--");
+		Util::ReduceStr(body, "<!-- %end% -->", "-->");
+		Util::ReduceStr(body, "%end% -->", "");
+		len = strlen(body);
+	}
+#endif
+
+	SendBodyResponse(body, len, contentType, true);
+}
+
+void WebProcessor::SendMultiFileResponse()
+{
+	debug("serving multiple files: %s", *m_url);
+
+	StringBuilder response;
+	char* filelist = strchr(m_url, '?');
+	*filelist++ = '\0';
+
+	Tokenizer tok(filelist, "+");
+	while (const char* filename = tok.Next())
+	{
+		BString<1024> diskFilename("%s%c%s", g_Options->GetWebDir(), PATH_SEPARATOR, filename);
+
+		CharBuffer body;
+		if (!FileSystem::LoadFileIntoBuffer(diskFilename, body, true))
+		{
+			warn("Web-Server: %s, Resource: /%s", ERR_HTTP_NOT_FOUND, filename);
+			SendErrorResponse(ERR_HTTP_NOT_FOUND, false);
+			return;
+		}
+
+		response.Append(body);
+	}
+
+	SendBodyResponse(response, response.Length(), DetectContentType(m_url), true);
 }
 
 const char* WebProcessor::DetectContentType(const char* filename)
